/*
 * FileLoader.java
 *
 * Created on 14 April 2010
 *
 * Version 1.1
 *
 * Copyright (c) Kieran Greer
 */

package org.jlog2.util;


import org.jlog2.*;

import java.io.*;
import java.net.URL;
import java.util.Vector;


/**
 * This class can be used to load in a file from a local or remote location.
 * @author Kieran Greer
 */
public class FileLoader
{
    /** The logger */
    protected static Logger logger;


    static
    {
        // get the logger
        logger = LoggerHandler.getLogger(FileLoader.class.getName());
        logger.setDebug(false);
    }


    /** Create a new instance of FileLoader */
    public FileLoader()
    {}

    /**
     * Get the contents of the file as a String.
     * @param fileName the name and location of the file to load.
     * @return the input stream read into a String, not a direct connection.
     * @throws java.lang.Exception any error.
     */
    public static String getInputStream(String fileName) throws Exception
    {
        String nextLine;                                //next line
        String fileString;                              //the input as a string
        URL url;                                        //file url
        InputStream inputStream;                        //input stream
        DataInputStream dataInputStream;                //data input stream
        BufferedReader br;                              //buffered reader
         
        try
        {
            url = new URL(fileName);
            inputStream = url.openStream();
            dataInputStream = new DataInputStream(inputStream);
        }
        catch (Exception ex)
        {
            //if file connection not supported then need to open standard
            //data input stream, but this will not have a lock on the file
            dataInputStream = new DataInputStream(new FileInputStream(fileName));
        }

        fileString = "";
        br = new BufferedReader(new InputStreamReader(dataInputStream));

        while ((nextLine = br.readLine()) != null)
        {
            fileString += (nextLine.trim() + "\n");
        }

        return fileString;
    }

    /**
     * Get the contents of the file as a buffered reader.
     * @param fileName the name and location of the file to load. This could be
     * at a remote location.
     * @return the input stream read into a buffered reader.
     * @throws java.lang.Exception any error.
     */
    public static BufferedReader getBufferedInputStream(String fileName) throws Exception
    {
        URL url;                                        //file url
        InputStream inputStream;                        //input stream
        DataInputStream dataInputStream;                //data input stream
        BufferedReader br;                              //buffered reader

        try
        {
            //filePath = fileName;
            url = new URL(fileName);
            inputStream = url.openStream();
            dataInputStream = new DataInputStream(inputStream);
        }
        catch (Exception ex)
        {
            //if file connection not supported then need to open standard
            //data input stream, but this will not have a lock on the file
            dataInputStream = new DataInputStream(new FileInputStream(fileName));
        }

        br = new BufferedReader(new InputStreamReader(dataInputStream));
        return br;
    }
    
    /**
     * Load a file and store in a buffer. This is for a local text-based file only.
     * @param filePath the path of the file to read.
     * @return the buffer that can read the file.
     * @throws java.lang.Exception any error.
     */
    public static BufferedReader loadFile(File filePath) throws Exception
    {
        BufferedReader buffReader;                      //buffered reader

        buffReader = new BufferedReader(new FileReader(filePath.toString()));
        return buffReader;
    }
    
    /**
     * Get the current directory the program is running in.
     * @return the directory path in String format.
     * @throws java.lang.Exception any error 
     */
    public static String getCurrentDirectory() throws Exception
    {
        String currentDir;              //current directory
        File dir;                       //current directory

        currentDir = null;
        dir = new File(".");
        currentDir = dir.getCanonicalPath();

        return currentDir;
    }
    
    /**
     * Get the list of directories stored in the specified directory.
     * @param dirPath the directory path.
     * @return a list of all directories stored under this directory.
     * @throws java.lang.Exception any error.
     */
    public static Vector getDirectoryList(String dirPath) throws Exception
    {
        int i;
        String nextFileName;                            //next file name
        File[] allFiles;                                //list of all files
        File fileDir;                                   //directory
        Vector fileList;                                //list of file names

        fileList = new Vector();
        fileDir = new File(dirPath);
        allFiles = fileDir.listFiles();

        if (allFiles != null)
        {
            for (i = 0; i < allFiles.length; i++)
            {
                nextFileName = allFiles[i].getName();
                if (allFiles[i].isDirectory())
                {
                    fileList.addElement(nextFileName);
                }
            }
        }

        return fileList;
    }
    
    /**
     * Get the list of files stored in the specified directory.
     * @param dirPath the directory path.
     * @return a list of all files in the directory.
     * @throws java.lang.Exception any error.
     */
    public static Vector getFileList(String dirPath) throws Exception
    {
        int i;
        String nextFileName;                            //next file name
        File[] allFiles;                                //list of all files
        File fileDir;                                   //directory
        Vector fileList;                                //list of file names

        fileList = new Vector();
        fileDir = new File(dirPath);
        allFiles = fileDir.listFiles();

        if (allFiles != null)
        {
            for (i = 0; i < allFiles.length; i++)
            {
                if (allFiles[i].isFile())
                {
                    nextFileName = allFiles[i].getName();
                    fileList.addElement(nextFileName);
                }
            }
        }

        return fileList;
    }
    
    /**
     * Copy the source input stream contents to the destination file.
     * @param sourceFile the file to copy.
     * @param destFile the file path to copy to
     * @throws java.lang.Exception any error.
     */
    public static void copyStream(File sourceFile, File destFile) throws Exception
    {
        InputStream is;                         //input stream
        OutputStream os;                        //output stream
        
        if(!destFile.exists()) destFile.createNewFile();
        is = new FileInputStream(sourceFile);
        os = new FileOutputStream(destFile);

        final byte[] buf = new byte[1024];
        int len = 0;
        while ((len = is.read(buf)) > 0) 
        {
            os.write(buf, 0, len);
        }

        is.close();
        os.close();
    }
    
    /**
     * Delete the file at the specified file path.
     * @param fileName the name of the file to delete.
     * @throws java.lang.Exception any error 
     */
    public static void deleteFile(String fileName) throws Exception
    {
        File toDelete;                          //file to delete

        toDelete = new File(fileName);
        toDelete.delete();
    }
}
